
/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 *
 * Contributor(s):
 *
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */

package com.sun.enterprise.shoal;

import com.sun.enterprise.ee.cms.core.CallBack;
import com.sun.enterprise.ee.cms.core.GMSException;
import com.sun.enterprise.ee.cms.core.GMSFactory;
import com.sun.enterprise.ee.cms.core.GroupManagementService;
import com.sun.enterprise.ee.cms.core.GroupManagementService.MemberType;
import com.sun.enterprise.ee.cms.core.JoinNotificationSignal;
import com.sun.enterprise.ee.cms.core.MessageSignal;
import com.sun.enterprise.ee.cms.core.ServiceProviderConfigurationKeys;
import com.sun.enterprise.ee.cms.core.Signal;
import com.sun.enterprise.ee.cms.core.GMSConstants;
import com.sun.enterprise.ee.cms.impl.client.JoinNotificationActionFactoryImpl;
import com.sun.enterprise.ee.cms.impl.client.MessageActionFactoryImpl;

import java.util.Properties;

/**
 * @author Rafael Barbosa
 */
public class TestShoal implements Runnable, CallBack {
    String serviceName = "service";
    GroupManagementService gms;
    private final Object sendMessagesSignal = new Object();
    private int nbOfMembers = 2;
    int total_msgs_received = 0;

    public TestShoal() {
        Properties props = new Properties();
        props.put(ServiceProviderConfigurationKeys.LOOPBACK.toString(), "true");
        gms = (GroupManagementService) GMSFactory.startGMSModule(System.getProperty("INSTANCEID"), "group", MemberType.CORE, props);

        try {
            gms.addActionFactory(new MessageActionFactoryImpl(this), serviceName);
            gms.addActionFactory(new JoinNotificationActionFactoryImpl(this));
            gms.join();
        } catch (GMSException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    public void run() {
        if (gms.getGroupHandle().getAllCurrentMembers().size() < nbOfMembers) {
            System.out.println("Waiting for all members to join...");
            synchronized (sendMessagesSignal) {
                try {
                    sendMessagesSignal.wait();
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
        }

        int num_msgs = 10000;
        byte[] payload = generatePayload();
        long log_interval = 1000;
        int total_msgs = 0;
        try {
            for (int i = 0; i < num_msgs; i++) {
                gms.getGroupHandle().sendMessage(serviceName, payload);
                total_msgs++;
                if (total_msgs % log_interval == 0) {
                    System.out.println("++ sent " + total_msgs);
                }
            }
        } catch (GMSException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
            synchronized (sendMessagesSignal) {
                try {
                    sendMessagesSignal.wait(60000);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
        // run is complete shutdown the instance
        gms.shutdown(GMSConstants.shutdownType.INSTANCE_SHUTDOWN);
    }

    byte[] generatePayload() {
        byte[] payload = new byte[128];
        for (int i = 0; i < payload.length; i++) {
            if (i % 2 == 0) {
                payload[i] = 1;
            }
        }
        return payload;
    }

    public void processNotification(Signal signal) {
        if (signal instanceof JoinNotificationSignal) {
            if (gms.getGroupHandle().getAllCurrentMembers().size() == nbOfMembers) {
                synchronized (sendMessagesSignal) {
                    sendMessagesSignal.notify();
                }
            }
        } else if (signal instanceof MessageSignal) {
            total_msgs_received++;
            if (total_msgs_received % 1000 == 0) {
                System.out.println("-- received " + total_msgs_received);
            }
        } else {
            System.err.println(new StringBuffer().append(serviceName)
                    .append(": Notification Received from:")
                    .append(signal.getMemberToken())
                    .append(":[")
                    .append(signal.toString())
                    .append("] has been processed")
                    .toString());
        }
    }

    public static void main(String[] args) {
        Thread t = new Thread(new TestShoal());
        t.start();
    }
}
